<!DOCTYPE html>
<html lang="en">

<head>
  <link rel="preconnect" href="https://fonts.gstatic.com/" crossorigin="anonymous">
  <link rel="preload" as="style" onload="this.rel='stylesheet'" href="https://fonts.googleapis.com/css2?family=Roboto:wght@100;300;400;500;700;900&display=swap">
  <link rel="preload" as="font" type="font/eot" href="/admin/static/materialdesignicons-webfont-DttUABo4.eot" crossorigin="anonymous">
  <link rel="preload" as="font" type="font/woff2" href="/admin/static/materialdesignicons-webfont-CYDMK1kx.woff2" crossorigin="anonymous">
  <link rel="preload" as="font" type="font/woff" href="/admin/static/materialdesignicons-webfont-CgCzGbLl.woff" crossorigin="anonymous">
  <link rel="preload" as="font" type="font/ttf" href="/admin/static/materialdesignicons-webfont-D3kAzl71.ttf" crossorigin="anonymous">

  <meta charset="UTF-8" />
  <link rel="icon" href="{{ favicon_image }}" />
  <meta name="viewport" content="width=device-width, initial-scale=1.0" />
  <title>{{ title }}</title>
  <script type="module" crossorigin src="/admin/static/index-D9axz5zK.js"></script>
  <link rel="stylesheet" crossorigin href="/admin/static/index-vlBToOhT.css">
</head>

<body>
  <div id="app"></div>
  <span id="settings" data-json="{{ settings_json }}"></span>
</body>

</html>
